from django.utils.decorators import method_decorator
from django.views.generic import DetailView
from django.shortcuts import render

from core.custom_annotations import group_required, in_groups
from core.models import Submission, Feedback

from .user_startpages import get_annotated_feedback_count, get_annotated_tutor_list


class SubmissionViewStudent(DetailView):

    template_name = 'core/s/single_submission.html'
    model = Submission

    @method_decorator(group_required('Students',))
    def dispatch(self, *args, **kwargs):
        return super(SubmissionViewStudent, self).dispatch(*args, **kwargs)

    def get_object(self):
        obj = Submission.objects.get(slug=self.kwargs['slug'])
        if in_groups(self.request.user, ('Students', )) and hasattr(obj, 'feedback') and obj.feedback.status == Feedback.ACCEPTED:
            obj.seen_by_student = True
            obj.save()
        return obj


class SubmissionViewReviewer(DetailView):

    template_name = 'core/r/single_submission.html'
    model = Submission

    @method_decorator(group_required('Reviewers',))
    def dispatch(self, *args, **kwargs):
        return super(SubmissionViewReviewer, self).dispatch(*args, **kwargs)

    def get_object(self):
        return Submission.objects.get(slug=self.kwargs['slug'])


@group_required('Reviewers')
def get_submission_list(request):
    context = {
        'submission_list': Submission.objects.all(),
        'submission_type_list': get_annotated_feedback_count(),
        'tutor_list': get_annotated_tutor_list(),
    }
    return render(request, 'core/r/student_submission_list.html', context)
